OutputSyntax

Enum OutputSyntax 

enum OutputSyntax {
    Implicit,
    Explicit,
}
How a relation header renders its output.

Variants



Implicit

Output columns are rendered as final visible output: => output_columns.



Explicit

Output columns are rendered as the direct output domain: +> columns, with |> order appended when an explicit emit mapping is present.