pub struct Relation<'a> {
pub name: Cow<'a, str>,
pub arguments: Option<Arguments<'a>>,
pub columns: Vec<Value<'a>>,
pub emit: Option<&'a EmitKind>,
pub advanced_extension: Option<&'a AdvancedExtension>,
pub children: Vec<Option<Relation<'a>>>,
}Fields
name: Cow<'a, str>arguments: Option<Arguments<'a>>Arguments to the relation, if any.
Nonemeans this relation does not take arguments, and the argument section is omitted entirely.Some(args)with both vectors empty means the relation takes arguments, but none are provided; this will print as_ => ....Some(args)with non-empty vectors will print as usual, with positional arguments first, then named arguments, separated by commas.
columns: Vec<Value<'a>>The columns emitted by this relation, pre-emit - the ‘direct’ column output.
emit: Option<&'a EmitKind>The emit kind, if any. If none, use the columns directly.
advanced_extension: Option<&'a AdvancedExtension>The advanced extension (enhancement and/or optimizations) attached to
this relation, if any. Mirrors the advanced_extension field carried
by standard relation types in the protobuf (Read, Filter, Project, etc…)
Extension relations (ExtensionLeaf, ExtensionSingle, ExtensionMulti)
do not carry this field and always set it to None.
children: Vec<Option<Relation<'a>>>The input relations.
Implementations
Sourceimpl Relation<'_>
impl Relation<'_>
Sourceimpl<'a> Relation<'a>
impl<'a> Relation<'a>
Sourcepub fn input_refs(&self) -> Vec<Value<'a>>
pub fn input_refs(&self) -> Vec<Value<'a>>
Create a vector of values that are references to the emitted outputs of this relation. “Emitted” here meaning the outputs of this relation after the emit kind has been applied.
This is useful for relations like Filter and Limit whose direct outputs are primarily those of its children (direct here meaning before the emit has been applied).
Sourceimpl<'a> Relation<'a>
impl<'a> Relation<'a>
fn from_filter<S: Scope>(rel: &'a FilterRel, ctx: &S) -> Self
fn from_project<S: Scope>(rel: &'a ProjectRel, ctx: &S) -> Self
pub fn from_rel<S: Scope>(rel: &'a Rel, ctx: &S) -> Self
fn from_extension_leaf<S: Scope>(rel: &'a ExtensionLeafRel, ctx: &S) -> Self
fn from_extension_single<S: Scope>(rel: &'a ExtensionSingleRel, ctx: &S) -> Self
fn from_extension_multi<S: Scope>(rel: &'a ExtensionMultiRel, ctx: &S) -> Self
fn from_extension<S: Scope>( ext_type: &'static str, decoded: Result<(String, ExtensionArgs), ExtensionError>, child_refs: Vec<Option<&'a Rel>>, ctx: &S, ) -> Self
Sourcefn from_aggregate<S: Scope>(rel: &'a AggregateRel, ctx: &S) -> Self
fn from_aggregate<S: Scope>(rel: &'a AggregateRel, ctx: &S) -> Self
Convert an AggregateRel to a Relation for textification.
The conversion follows this logic:
- Arguments: Group-by expressions (as Value::Expression)
- Columns: All possible outputs in order:
- First: Group-by field references (Value::Reference)
- Then: Aggregate function measures (Value::AggregateFunction)
- Emit: Uses the relation’s emit mapping to select which outputs to display
- Children: The input relation